.comment-component-content{
    max-width: 550px;
    max-height: 300px;
    overflow-y: auto;
    white-space: pre;
    cursor: pointer;
    padding: 5px;
    box-sizing: border-box;
    font-size: 14px;
    font-family: sans-serif;
    font-weight: bolder;
    text-align: justify;
    background-color: rgba(77, 77, 77, 0.9);
    color: rgb(255,255,255,0.9);
}