/*-----------------------------------------------------------------------------
| Copyright (c) XpressAI GK and Jupyter Development Team.
| Distributed under the terms of the Modified BSD License.
|----------------------------------------------------------------------------*/

.jp-DebuggerWidget {
  color: var(--jp-ui-font-color1);
  background: var(--jp-layout-color1);
  font-size: 14px;
  align-items: center;
  justify-content: left;
  text-align: left;
  overflow-y: auto;
}

.jp-debugger-toolbar-panel {
  flex: 0 0 auto;
}

input[type=number] {
  line-height: 35px;
}

input[type=number]::-webkit-inner-spin-button {
  width: 35px;
  height: 35px;
}

/* Remove outer resizing of run dialog */
.jp-Dialog-content {
  resize: none !important;
}

/* Fix arrow position of HTMLSelect tag on run dialog */
.jp-Dialog-content .f1ozlkqi {
  position: relative;
}

.alignToggle {
  display: flex;
  flex-direction: row;
  align-items: center;
  justify-content: flex-start;
}

.alignToggle span {
  display: inline-block;
  padding: 0;
  margin: 0;
}